from __future__ import annotations

from silx.gui import qt

from ewoks3dxrd.models import GridIndexParameters

from ..common.tolerance_selector import ToleranceSelector
from ..common.tuple_input_widget import TupleInputWidget
from .constants import Symmetry


class GridIndexParameterGroupBox(qt.QGroupBox):

    def __init__(self, parent: qt.QWidget | None = None, **kwargs) -> None:
        super().__init__("Grid Index Settings", parent=parent, **kwargs)
        layout = qt.QFormLayout(self)

        self._npks = qt.QSpinBox()
        self._npks.setRange(1, 10000)
        self._npks.setValue(100)
        self._npks.setToolTip("Minimal number of peaks for a grain to be kept")

        self._dstol = qt.QDoubleSpinBox()
        self._dstol.setRange(0.0001, 0.01)
        self._dstol.setSingleStep(0.001)
        self._dstol.setDecimals(4)
        self._dstol.setToolTip(
            "Choose Reciprocal distance tolerance. ex 10e-4 to 10e-3"
        )
        self._dstol.setValue(0.004)

        self._ring1 = TupleInputWidget(placeHolderText="e.g. 1, 0", Type=int)
        self._ring1.setToolTip("Ring 1 indices as comma-separated integers.")

        self._ring2 = TupleInputWidget(placeHolderText="e.g. 0", Type=int)
        self._ring2.setToolTip("Ring 2 indices as comma-separated integers.")

        self._tolangle = qt.QDoubleSpinBox()
        self._tolangle.setRange(0.0001, 10.0)
        self._tolangle.setSingleStep(0.01)
        self._tolangle.setToolTip("Tolerance angle in degrees.")
        self._tolangle.setValue(0.50)
        degSymbol = "\u00b0"
        self._tolangle.setSuffix(f" {degSymbol}")

        self._toldist = qt.QDoubleSpinBox()
        self._toldist.setRange(0.0, 200.0)
        self._toldist.setToolTip(
            "Tolerance Distance, usually in range of 0.5 to 2 times detector pixel width."
        )
        self._toldist.setValue(100.0)
        self._toldist.setSuffix(" µm")

        self._toleranceGroup = ToleranceSelector()

        self._symmetry = qt.QComboBox()
        self._symmetry.addItems([e.value for e in Symmetry])
        self._symmetry.setToolTip("Optional. Symmetry group.")

        layout.addRow("Number of Peaks", self._npks)
        layout.addRow("Reciprocal Tolerance", self._dstol)
        layout.addRow("Ring 1", self._ring1)
        layout.addRow("Ring 2", self._ring2)
        layout.addRow("Tolerance Angle", self._tolangle)
        layout.addRow("Tolerance Distance", self._toldist)
        layout.addRow("Crystal Symmetry", self._symmetry)
        layout.addRow("Tolerance Tuning", self._toleranceGroup)

    def getGridIndexParameters(self) -> GridIndexParameters:
        try:
            self._validate()
            return GridIndexParameters(
                NPKS=self._npks.value(),
                DSTOL=self._dstol.value(),
                RING1=self._ring1.getValue(),
                RING2=self._ring2.getValue(),
                tolangle=self._tolangle.value(),
                toldist=self._toldist.value(),
                TOLSEQ=self._toleranceGroup.getValue(),
                SYMMETRY=self._symmetry.currentText(),
            )
        except Exception as e:
            raise ValueError(e)

    def setGridIndexParameters(self, params: GridIndexParameters):
        self._npks.setValue(params.NPKS)
        self._dstol.setValue(params.DSTOL)
        self._ring1.setValue(params.RING1)
        self._ring2.setValue(params.RING2)
        self._tolangle.setValue(params.tolangle)
        self._toleranceGroup.setValue(params.TOLSEQ)
        self._symmetry.setCurrentText(params.SYMMETRY)

    def _validate(self):
        if not self._ring1.getValue():
            raise ValueError("Ring1 must not be empty")
        if not self._ring2.getValue():
            raise ValueError("Ring2 must not be empty")



from __future__ import annotations

from silx.gui import qt
from silx.gui.plot import Plot2D


class Ewoks3DXRDPlot2D(Plot2D):
    def __init__(self, parent: qt.QWidget | None = None) -> None:
        super().__init__(parent=parent, backend="gl")
        self.setBackgroundColor("white")
        self.setKeepDataAspectRatio(True)
        self.setAxesMargins(0.06, 0.06, 0.06, 0.06)
        self.setGraphGrid(True)
        self.setInteractiveMode(mode="pan")



from __future__ import annotations

from silx.gui import qt
from silx.gui.colors import Colormap
from silx.io.utils import DataUrl

from ewoks3dxrd.nexus.peaks import read_peaks_attributes
from ewoks3dxrd.nexus.utils import group_exists
from ewoks3dxrd.tasks.filter_by_intensity import FilterByIntensity, Inputs

from .common.ewoks3dxrd_widget import Ewoks3DXRDWidget
from .common.peak_filter_plot2d import PeakFilterPlot2D
from .common.process_name_line_edit import ProcessNameLineEdit


class OWIntensityFilter(Ewoks3DXRDWidget, ewokstaskclass=FilterByIntensity):
    name = "Intensity Filter"
    description = "Filter peaks based on the Intensity."
    icon = "icons/filter-invert.svg"

    _ewoks_inputs_to_hide_from_orange = (
        "overwrite",
        "intensity_frac",
        "process_group_name",
    )

    def __init__(self):
        super().__init__()

        self._settingsPanelWidget = qt.QWidget(self)
        settingLayout = qt.QVBoxLayout(self._settingsPanelWidget)

        self._intensityFrac = qt.QDoubleSpinBox()
        self._intensityFrac.setSingleStep(0.0001)
        self._intensityFrac.setRange(0.001, 1.0)
        self._intensityFrac.setValue(0.9999)
        self._intensityFrac.setDecimals(4)
        hbox = qt.QHBoxLayout()
        label = qt.QLabel("Intensity Fraction")
        hbox.addWidget(label)
        hbox.addWidget(self._intensityFrac)
        settingLayout.addLayout(hbox)

        executeOverWriteLayout = qt.QFormLayout()
        self._processNameLineEdit = ProcessNameLineEdit(
            Inputs.model_fields["process_group_name"].default
        )
        self._inputPeaksUrl = qt.QLineEdit()
        self._overwrite = qt.QCheckBox()
        self._filterPeaksBtn = qt.QPushButton("Filter Peaks")
        executeOverWriteLayout.addRow("Incoming Data URL", self._inputPeaksUrl)
        executeOverWriteLayout.addRow(
            self._processNameLineEdit.label, self._processNameLineEdit
        )
        executeOverWriteLayout.addRow("Overwrite", self._overwrite)
        executeOverWriteLayout.addRow(self._filterPeaksBtn)
        settingLayout.addLayout(executeOverWriteLayout)
        self._filterPeaksBtn.clicked.connect(self.execute_ewoks_task)

        self.addControlWidget(self._settingsPanelWidget)
        self._plot = PeakFilterPlot2D(self)
        self.addMainWidget(self._plot)
        self.registerInput(
            "lattice_filtered_data_url",
            self._validateInputUrl,
            self._inputPeaksUrl.setText,
        )
        self.registerInput(
            "intensity_frac", self._intensityFrac.value, self._intensityFrac.setValue
        )
        self.registerInput(
            "process_group_name",
            self._processNameLineEdit.getText,
            self._processNameLineEdit.setText,
        )
        self.registerInput(
            "overwrite", self._overwrite.isChecked, self._overwrite.setChecked
        )
        self._restoreDefaultInputs()
        self._plotIncomingPeaks()

    def handleNewSignals(self):
        input_url = self.get_task_input_value("lattice_filtered_data_url", None)
        if input_url:
            self._inputPeaksUrl.setText(input_url)
            self._validateInputUrl()
            self._plotIncomingPeaks()

    def _plotIncomingPeaks(self):
        inputUrl = self._inputPeaksUrl.text().strip()
        if not inputUrl:
            return
        data_url = DataUrl(inputUrl)
        nexus_file_path, lattice_data_group_path = (
            data_url.file_path(),
            data_url.data_path(),
        )
        lattice_3d_peaks = read_peaks_attributes(
            filename=nexus_file_path,
            process_group=lattice_data_group_path,
        )

        self._plot.addUnfilteredScatter(
            x=lattice_3d_peaks["ds"],
            y=lattice_3d_peaks["sum_intensity"],
            value=len(lattice_3d_peaks["ds"]) * [1],
            colormap=Colormap(name="viridis", autoscaleMode="percentile_1_99"),
        )
        self._plot.setGraphXLabel("Reciprocal distance (ds)")
        self._plot.setGraphYLabel("Sum Intensity")
        self._plot.setYAxisLogarithmic(True)
        self._plot.resetZoom()

    def handleSuccessfulExecution(self):
        if self.get_task_output_value("intensity_filtered_data_url", None) is None:
            return

        data_url = DataUrl(self.get_task_output_value("intensity_filtered_data_url"))
        nexus_file_path, lattice_data_group_path = (
            data_url.file_path(),
            data_url.data_path(),
        )
        intensity_3d_peaks = read_peaks_attributes(
            filename=nexus_file_path,
            process_group=lattice_data_group_path,
        )
        self._plot.addFilteredScatter(
            x=intensity_3d_peaks["ds"],
            y=intensity_3d_peaks["sum_intensity"],
            value=intensity_3d_peaks["Number_of_pixels"],
            colormap=Colormap(
                name="viridis", normalization="log", autoscaleMode="percentile_1_99"
            ),
        )
        self._plot.setGraphXLabel("Reciprocal distance (ds)")
        self._plot.setGraphYLabel("Sum Intensity")
        self._plot.resetZoom()

    def _validateInputUrl(self) -> str:
        input_peaks_url = DataUrl(self._inputPeaksUrl.text().strip())
        group_exists(
            filename=input_peaks_url.file_path(),
            data_group_path=input_peaks_url.data_path(),
        )
        return input_peaks_url.path()



from silx.gui import qt

from ewoks3dxrd.models import GridIndexParameters

from .grid_index_parameter_group_box import GridIndexParameterGroupBox


class GridIndexSettings(qt.QWidget):
    sigGridSettingsChanged = qt.Signal()

    def __init__(self, parent=None):
        super().__init__(parent)
        self.setWindowTitle("Grid Index Config")
        self.setSizePolicy(qt.QSizePolicy.Expanding, qt.QSizePolicy.Expanding)
        layout = qt.QVBoxLayout()

        _gridSearchGroup = qt.QGroupBox("Sample and Beam Info")

        self._sampleWidth = qt.QSpinBox()
        self._sampleWidth.setRange(1, 10000)
        self._sampleWidth.setValue(1200)
        self._sampleWidth.setSuffix(" µm")
        self._sampleWidth.setToolTip("Provide width of sample.")

        self._sampleHeight = qt.QSpinBox()
        self._sampleHeight.setRange(1, 10000)
        self._sampleHeight.setValue(1200)
        self._sampleHeight.setSuffix(" µm")
        self._sampleHeight.setToolTip("Provide height of sample.")

        self._beamIlluminationRange = qt.QSpinBox()
        self._beamIlluminationRange.setRange(1, 10000)
        self._beamIlluminationRange.setValue(200)
        self._beamIlluminationRange.setSuffix(" µm")
        self._beamIlluminationRange.setToolTip("Provide beam height.")

        self._grainSpottingWidth = qt.QSpinBox()
        self._grainSpottingWidth.setRange(1, 10000)
        self._grainSpottingWidth.setValue(100)
        self._grainSpottingWidth.setSuffix(" µm")
        self._grainSpottingWidth.setToolTip("Provide 1 or 2 times detector pixel size.")

        gridSearchLayout = qt.QFormLayout(_gridSearchGroup)
        gridSearchLayout.addRow("Sample Width", self._sampleWidth)
        gridSearchLayout.addRow("Sample Height", self._sampleHeight)
        gridSearchLayout.addRow("Beam Width", self._beamIlluminationRange)
        gridSearchLayout.addRow("Grid Step", self._grainSpottingWidth)

        layout.addWidget(_gridSearchGroup)
        self._gridIndexParametergroup = GridIndexParameterGroupBox(self)
        layout.addWidget(self._gridIndexParametergroup)
        self.setLayout(layout)

    def getGridIndexParameters(self) -> dict:
        return self._gridIndexParametergroup.getGridIndexParameters().model_dump()

    def getGridXLimit(self) -> int:
        return self._sampleWidth.value()

    def getGridYLimit(self) -> int:
        return self._sampleHeight.value()

    def getGridZLimit(self) -> int:
        return self._beamIlluminationRange.value()

    def getGridStep(self) -> int:
        return self._grainSpottingWidth.value()

    def setGridIndexParameters(self, value: dict):
        self._gridIndexParametergroup.setGridIndexParameters(
            GridIndexParameters(**value)
        )

    def setGridXLimit(self, value: int):
        self._sampleWidth.setValue(value)

    def setGridYLimit(self, value: int):
        self._sampleHeight.setValue(value)

    def setGridZLimit(self, value: int):
        self._beamIlluminationRange.setValue(value)

    def setGridStep(self, value: int):
        self._grainSpottingWidth.setValue(value)



from __future__ import annotations

import os.path
from pathlib import Path
from typing import Sequence, Tuple, Union

from ewokscore import Task
from ImageD11 import grain as grainMod
from pydantic import Field

from ..io import find_omega_slop
from ..models import InputsWithOverwrite, SampleConfig
from ..nexus.grains import save_nexus_grain_process
from ..nexus.peaks import save_column_file_as_ascii
from ..nexus.utils import (
    check_throw_write_data_group_url,
    get_data_url_paths,
    get_entry_name,
)
from ..tmp_files import tmp_grain_processing_files
from ..utils import refine_grains


class Inputs(InputsWithOverwrite):
    folder_file_config: SampleConfig
    indexed_grain_data_url: str = Field(
        description="Path to NXprocess group containing the indexed grains"
    )
    intensity_filtered_data_url: str = Field(
        description="Path to NXprocess group containing the filtered peaks used for the indexing"
    )
    hkl_tols: Sequence[float] = Field(
        description="Decreasing sequence of hkl tolerances. Will be used for iterative refinement (one after the other).",
        examples=[(0.3, 0.2, 0.1)],
    )
    minpks: int = Field(
        ge=0,
        description="Minimal number of peaks for the grain to be kept after iterative refinement.",
    )
    intensity_fine_filtered_data_url: Union[str, None] = Field(
        default=None,
        description="Path to NXprocess group containing the peaks to use to refine the grains finely at the end of the iterative refinement",
    )
    intensity_two_theta_range: Tuple[float, float] = Field(
        default=(0.0, 180.0), description="Range of two theta to use when refining"
    )
    symmetry: str = Field(
        default="cubic", description="Lattice symmetry used to further refine grains"
    )
    analyse_folder: Union[str, Path, None] = None
    lattice_file: str = Field(
        description="Path to the file (`.par` or `.cif`) containing lattice parameters and space group information forwarded by lattice filter task"
    )


class MakeGrainMap(Task, input_model=Inputs, output_names=["make_map_data_url"]):
    """
    Does an iterative refinement based on `hkl_tols` followed by a fine refinement on the indexed grains

    Outputs:
    - `ascii_grain_map_file`: file where the refined grains are saved
    """

    def run(self):
        inputs = Inputs(**self.get_input_values())
        nexus_file_path, intensity_filtered_data_path = get_data_url_paths(
            inputs.intensity_filtered_data_url
        )

        analyse_folder = (
            Path(inputs.analyse_folder)
            if inputs.analyse_folder
            else Path(nexus_file_path).parent
        )
        flt_pks_file = analyse_folder / "flt_3d_peaks.flt"
        save_column_file_as_ascii(
            inputs.intensity_filtered_data_url,
            flt_pks_file,
        )

        if inputs.intensity_fine_filtered_data_url is None:
            fine_flt_pks_file = flt_pks_file
        else:
            fine_flt_pks_file = analyse_folder / "fine_flt_3d_peaks.flt"
            save_column_file_as_ascii(
                inputs.intensity_fine_filtered_data_url,
                fine_flt_pks_file,
            )

        entry_name = get_entry_name(intensity_filtered_data_path)
        check_throw_write_data_group_url(
            overwrite=inputs.overwrite,
            filename=nexus_file_path,
            data_group_path=f"{entry_name}/make_map_grains",
        )

        geo_par_url = f"{nexus_file_path}::{entry_name}/geometry_updated_peaks"

        omega_slop = find_omega_slop(inputs.folder_file_config)
        with tmp_grain_processing_files(
            ubi_init_data_url=inputs.indexed_grain_data_url,
            geo_init_data_url=geo_par_url,
            lattice_parameter_file=inputs.lattice_file,
        ) as (ubi_file, par_file):
            for tol in inputs.hkl_tols:
                iterative_refined_grains = refine_grains(
                    tolerance=tol,
                    intensity_tth_range=inputs.intensity_two_theta_range,
                    omega_slop=omega_slop,
                    parameter_file=par_file,
                    filtered_peaks_file=flt_pks_file,
                    ubi_file=ubi_file,
                    symmetry=inputs.symmetry,
                )
                iterative_refined_grains.savegrains(ubi_file, sort_npks=True)

            refined_grains = grainMod.read_grain_file(ubi_file)
            grains_filtered = [
                grain for grain in refined_grains if float(grain.npks) > inputs.minpks
            ]
            grainMod.write_grain_file(filename=ubi_file, list_of_grains=grains_filtered)

            # fine refinement
            fine_refined_grains = refine_grains(
                tolerance=inputs.hkl_tols[-1],
                intensity_tth_range=inputs.intensity_two_theta_range,
                omega_slop=omega_slop,
                parameter_file=par_file,
                filtered_peaks_file=fine_flt_pks_file,
                ubi_file=ubi_file,
                symmetry=inputs.symmetry,
            )

        ubi_grain_file = os.path.join(analyse_folder, "refined_grains_map_file.flt")
        fine_refined_grains.savegrains(ubi_grain_file, sort_npks=True)
        grains_list = grainMod.read_grain_file(filename=ubi_grain_file)

        makemap_grains_group_name = save_nexus_grain_process(
            filename=nexus_file_path,
            entry_name=entry_name,
            process_name="make_map_grains",
            config_settings={
                "intensity_tth_range": inputs.intensity_two_theta_range,
                "omega_slop": omega_slop,
                "hkl_tol": inputs.hkl_tols,
                "grains_came_from": inputs.indexed_grain_data_url,
                "peaks_came_from": inputs.intensity_filtered_data_url,
                "fine_peaks_came_from": inputs.intensity_fine_filtered_data_url,
                "minpks": inputs.minpks,
                "hkl_tol_seq": inputs.hkl_tols[-1],
                "symmetry": inputs.symmetry,
            },
            grains=grains_list,
            overwrite=inputs.overwrite,
        )
        self.outputs.make_map_data_url = makemap_grains_group_name



import h5py
from silx.io.url import DataUrl

from ewoks3dxrd.tasks.filter_by_index import FilterByIndexer


def test_execution(nexus_3dxrd_file):
    inputs = {
        "intensity_filtered_data_url": f"{nexus_3dxrd_file}::/1.1/intensity_filtered_inner_peaks",
        "rings": (0, 1),
        "ds_tol": 0.01,
        "wavelength": 0.28457,
        "lattice_file": "/data/projects/id03_3dxrd/expt/PROCESSED_DATA/Fe.par",
    }

    task = FilterByIndexer(inputs=inputs)
    task.execute()
    output_url = DataUrl(task.outputs["indexer_filtered_data_url"])

    with h5py.File(output_url.file_path(), "r") as h5file:
        process_group = h5file[output_url.data_path()]
        assert isinstance(process_group, h5py.Group)
        assert isinstance(process_group["peaks/Number_of_pixels"], h5py.Dataset)



from silx.gui import qt
from .double_slider import QDoubleSlider


class GrainSizeSlider(QDoubleSlider):
    def __init__(self, orientation=qt.Qt.Orientation.Horizontal, parent=None):
        super().__init__(orientation, parent)
        self.setRange(0.00002, 0.0002)
        self.setFloatValue(0.0001)



from tqdm import tqdm


class TqdmProgressCallback(tqdm):
    def __init__(self, *args, TaskInstance=None, **kwargs):
        super().__init__(*args, **kwargs)
        self.TaskInstance = TaskInstance
        self.finished_count = 0

    def update(self, n: int = 1):
        super().update(n)
        self.finished_count += n
        if self.TaskInstance:
            self.TaskInstance.progress = 100.0 * (self.finished_count / self.total)



import h5py
from silx.io.url import DataUrl

from ewoks3dxrd.tasks.filter_by_lattice import FilterByLattice
from ewoks3dxrd.io import save_geometry_and_lattice_file


def test_execution(nexus_3dxrd_file, tmp_path):
    par_file = tmp_path / "Al.par"
    save_geometry_and_lattice_file(
        tmp_path / "Al.par",
        {},
        {
            "lattice_parameters": [4.050, 4.050, 4.050, 90.0, 90.0, 90.0],
            "lattice_space_group": 225,
        },
    )

    inputs = {
        "geometry_updated_data_url": f"{nexus_3dxrd_file}::/1.1/geometry_updated_peaks",
        "lattice_file": str(par_file),
        "reciprocal_dist_tol": 0.025,
    }

    task = FilterByLattice(inputs=inputs)
    task.execute()
    output_url = DataUrl(task.outputs["lattice_filtered_data_url"])

    with h5py.File(output_url.file_path(), "r") as h5file:
        process_group = h5file[output_url.data_path()]
        assert isinstance(process_group, h5py.Group)
        assert isinstance(process_group["peaks/Number_of_pixels"], h5py.Dataset)



import os
import sys

if sys.version_info >= (3, 9):
    from importlib.resources import files as resource_files
else:
    from importlib_resources import files as resource_files

from pathlib import Path

from ewokscore import execute_graph
from silx.io.utils import DataUrl

from orangecontrib.ewoks3dxrd import tutorials

from ..nexus.grains import read_grains_attributes
from ..nexus.peaks import read_peaks_attributes
from ..nexus.utils import get_data_url_paths, group_exists


def test_segmentation_3dpeaks_id03_workflow(id03_inp_config):
    assert os.environ["OMP_NUM_THREADS"] == "1"
    segmenter_config = {
        "threshold": 70,
        "smooth_sigma": 1.0,
        "bgc": 0.9,
        "min_px": 3,
        "offset_threshold": 100,
        "ratio_threshold": 150,
    }

    file_cor_config = {
        "bg_file": id03_inp_config.get("bg_file"),
        "mask_file": id03_inp_config.get("mask_file"),
        "flat_file": None,
        "dark_file": None,
    }
    sample_folder_config = {
        "detector": id03_inp_config.get("detector"),
        "omega_motor": id03_inp_config.get("omega_motor"),
        "master_file": id03_inp_config["master_file"],
        "scan_number": id03_inp_config["scan_number"],
        "analyse_folder": id03_inp_config.get("analyse_folder"),
    }

    inputs = [
        {
            "name": "folder_config",
            "value": sample_folder_config,
            "id": "segScan",
        },
        {
            "name": "segmenter_algo_params",
            "value": segmenter_config,
            "id": "segScan",
        },
        {
            "name": "correction_files",
            "value": file_cor_config,
            "id": "segScan",
        },
        {
            "name": "correction_files",
            "value": id03_inp_config["spline_file"],
            "id": "DetCor",
        },
        {
            "name": "geometry_par_file",
            "value": "/data/projects/id03_3dxrd/id03_expt/PROCESSED_DATA/geometry_closer.par",
            "id": "GeoUp",
        },
        {
            "name": "lattice_file",
            "value": "/data/projects/id03_3dxrd/id03_expt/PROCESSED_DATA/Al.par",
            "id": "LatFilt",
        },
        {
            "name": "reciprocal_dist_max",
            "value": 0.737,
            "id": "LatFilt",
        },
        {
            "name": "reciprocal_dist_tol",
            "value": 0.025,
            "id": "LatFilt",
        },
        {
            "name": "process_group_name",
            "value": "inner_rings",
            "id": "LatFilt",
        },
        {
            "name": "intensity_frac",
            "value": 0.999,
            "id": "IntFilt",
        },
        {
            "name": "process_group_name",
            "value": "intensity_inner_rings",
            "id": "IntFilt",
        },
        {
            "name": "lattice_file",
            "value": "/data/projects/id03_3dxrd/id03_expt/PROCESSED_DATA/Al.par",
            "id": "AllLatFilt",
        },
        {
            "name": "reciprocal_dist_tol",
            "value": 0.025,
            "id": "AllLatFilt",
        },
        {
            "name": "process_group_name",
            "value": "all_rings",
            "id": "AllLatFilt",
        },
        {
            "name": "intensity_frac",
            "value": 0.999,
            "id": "AllIntFilt",
        },
        {
            "name": "process_group_name",
            "value": "intensity_all_rings",
            "id": "AllIntFilt",
        },
        {
            "name": "rings",
            "value": (0, 1, 2),
            "id": "IndexerFilt",
        },
        {
            "name": "process_group_name",
            "value": "inner_rings_indexer_filter",
            "id": "IndexerFilt",
        },
        {"name": "grid_index_parameters", "value": {"NPKS": 44}, "id": "GridIndex"},
        {
            "name": "minpks",
            "value": 0,
            "id": "GrainMap",
        },
        {
            "name": "hkl_tols",
            "value": (0.01,),
            "id": "GrainMap",
        },
    ]
    workflow_file = resource_files(tutorials).joinpath("id03_3dxrd_workflow.json")

    outputs = execute_graph(
        str(workflow_file), inputs=inputs, outputs=[{"all": True}], merge_outputs=False
    )
    assert outputs is not None
    detector_spatial_data_path = outputs["DetCor"]["spatial_corrected_data_url"]
    data_url = DataUrl(detector_spatial_data_path)
    nexus_file_path = data_url.file_path()
    detector_spatial_corrected_group = data_url.data_path()
    assert group_exists(nexus_file_path, detector_spatial_corrected_group)
    detector_spatial_peaks = read_peaks_attributes(
        filename=nexus_file_path, process_group=detector_spatial_corrected_group
    )
    peak_x_position = detector_spatial_peaks["f_raw"]
    assert len(peak_x_position) == 30281

    lattice_filtered_data_path = outputs["LatFilt"]["lattice_filtered_data_url"]
    data_url = DataUrl(lattice_filtered_data_path)
    lattice_filtered_data_group = data_url.data_path()
    assert group_exists(nexus_file_path, lattice_filtered_data_group)
    lattice_file = outputs["LatFilt"]["copied_lattice_file"]
    assert Path(lattice_file).exists()
    intensity_filtered_data_path = outputs["IntFilt"]["intensity_filtered_data_url"]
    data_url = DataUrl(intensity_filtered_data_path)
    intensity_filtered_data_group = data_url.data_path()
    assert group_exists(nexus_file_path, intensity_filtered_data_group)
    intensity_filtered_peaks = read_peaks_attributes(
        filename=nexus_file_path, process_group=intensity_filtered_data_group
    )
    peak_x_position = intensity_filtered_peaks["f_raw"]
    assert len(peak_x_position) == 18794

    lattice_filtered_data_path = outputs["AllLatFilt"]["lattice_filtered_data_url"]
    data_url = DataUrl(lattice_filtered_data_path)
    lattice_filtered_data_group = data_url.data_path()
    assert group_exists(nexus_file_path, lattice_filtered_data_group)
    lattice_file = outputs["LatFilt"]["copied_lattice_file"]
    assert Path(lattice_file).exists()
    intensity_filtered_data_path = outputs["AllIntFilt"]["intensity_filtered_data_url"]
    data_url = DataUrl(intensity_filtered_data_path)
    intensity_filtered_data_group = data_url.data_path()
    assert group_exists(nexus_file_path, intensity_filtered_data_group)
    intensity_filtered_peaks = read_peaks_attributes(
        filename=nexus_file_path, process_group=intensity_filtered_data_group
    )
    peak_x_position = intensity_filtered_peaks["f_raw"]
    assert len(peak_x_position) == 27304

    indexer_filtered_data_path = outputs["IndexerFilt"]["indexer_filtered_data_url"]
    data_url = DataUrl(indexer_filtered_data_path)
    inner_rings_indexer_filter_data_group = data_url.data_path()
    assert group_exists(nexus_file_path, inner_rings_indexer_filter_data_group)
    index_filtered_peaks = read_peaks_attributes(
        filename=nexus_file_path, process_group=inner_rings_indexer_filter_data_group
    )
    peak_x_position = index_filtered_peaks["f_raw"]
    assert len(peak_x_position) == 18794

    grid_index_data_path = outputs["GridIndex"]["grid_indexed_grain_data_url"]
    data_url = DataUrl(grid_index_data_path)
    grid_index_grains = data_url.data_path()
    assert group_exists(nexus_file_path, grid_index_grains)

    nexus_file_path, ubi_data_group_path = get_data_url_paths(
        outputs["GrainMap"]["make_map_data_url"]
    )
    assert Path(nexus_file_path).exists()
    dict_grains = read_grains_attributes(
        filename=nexus_file_path, process_group=ubi_data_group_path
    )
    assert dict_grains["UBI"].shape == (45, 3, 3)



from __future__ import annotations

from typing import Any, Optional, Sequence

from ewokscore import Task
from ImageD11 import columnfile as PeakColumnFile
from ImageD11 import indexing
from ImageD11.unitcell import unitcell as UnitCell
from pydantic import Field

from ..io import read_lattice_cell_data
from ..models import InputsWithOverwrite
from ..nexus.grains import save_nexus_grain_process
from ..nexus.peaks import read_peaks_attributes
from ..nexus.utils import (
    check_throw_write_data_group_url,
    get_data_url_paths,
    get_entry_name,
)


class Inputs(InputsWithOverwrite):
    intensity_filtered_data_url: str = Field(
        description="Path to NXprocess group containing the intensity filtered peaks"
    )
    rings: Sequence[int] = Field(
        description="Indices of rings used for generating UBI. Two are usually enough, three in some rare cases.",
        examples=[(0, 1)],
    )
    scoring_rings: Sequence[int] = Field(
        description="Indices of the rings used for scoring. These must contain the indices used for indexing.",
        examples=[(0, 1, 2)],
    )
    max_grains: Optional[int] = Field(
        default=None, description="Limits the number of found grains."
    )
    reciprocal_dist_tol: Optional[float] = Field(
        default=None, description="Reciprocal distance tolerance value."
    )
    hkl_tols: Optional[Sequence[float]] = Field(
        default=None,
        description="Miller indices tolerances.",
        examples=[(0.01, 0.02, 0.03, 0.04)],
    )
    min_pks_frac: Optional[Sequence[float]] = Field(
        default=None, description="Minimal peaks fraction to iterate over."
    )
    cosine_tol: Optional[float] = Field(
        default=None,
        description="tolerance value used in the Indexer convergence scheme for finding pairs of peaks to make an orientation",
    )
    lattice_file: str = Field(
        description="Path to the file (`.par` or `.cif`) containing lattice parameters and space group information forwarded by lattice filter task"
    )
    wavelength: float = Field(
        description="wavelength used in the geometry transformation task",
    )

    def imageD11_indexing_parameters(self) -> dict[str, Any]:
        indexing_params: dict[str, Any] = {
            "wavelength": self.wavelength,
        }

        if self.max_grains:
            indexing_params["max_grains"] = self.max_grains
        if self.reciprocal_dist_tol:
            indexing_params["dstol"] = self.reciprocal_dist_tol
        if self.hkl_tols:
            indexing_params["hkl_tols"] = self.hkl_tols
        if self.min_pks_frac:
            indexing_params["fracs"] = self.min_pks_frac
        if self.cosine_tol:
            indexing_params["cosine_tol"] = self.cosine_tol

        return indexing_params


class IndexGrains(Task, input_model=Inputs, output_names=["indexed_grain_data_url"]):
    """
    From 3D peaks, finds grains' UBI  matrices and stores them in both ASCII format and NeXus (.h5) format.

    Outputs:

    - `indexed_grain_data_url` (str): Data path to 'NxProcess group data' Grains that stores generated UBI
    """

    def run(self):
        inputs = Inputs(**self.get_input_values())
        nexus_file_path, intensity_filtered_data_group_path = get_data_url_paths(
            inputs.intensity_filtered_data_url
        )

        if len(inputs.rings) < 2:
            raise ValueError(
                f"Indexing needs at least two ring indices in `rings`. Got {inputs.rings}"
            )

        entry_name = get_entry_name(intensity_filtered_data_group_path)
        indexed_ubi_data_group = f"{entry_name}/indexed_grains"
        check_throw_write_data_group_url(
            overwrite=inputs.overwrite,
            filename=nexus_file_path,
            data_group_path=indexed_ubi_data_group,
        )

        intensity_filtered_peaks = read_peaks_attributes(
            filename=nexus_file_path, process_group=intensity_filtered_data_group_path
        )
        cf = PeakColumnFile.colfile_from_dict(intensity_filtered_peaks)

        lattice_parameter = read_lattice_cell_data(inputs.lattice_file)
        unit_cell = UnitCell(
            lattice_parameters=lattice_parameter.lattice_parameters,
            symmetry=str(lattice_parameter.space_group),
        )

        unit_cell.makerings(limit=cf.ds.max())

        indexing_params = inputs.imageD11_indexing_parameters()

        grains, _ = indexing.do_index(
            cf=cf,
            forgen=inputs.rings,
            foridx=inputs.scoring_rings,
            unitcell=unit_cell,
            **indexing_params,
        )

        ubi_grains_group_name = save_nexus_grain_process(
            filename=nexus_file_path,
            entry_name=entry_name,
            process_name="indexed_grains",
            config_settings={
                **indexing_params,
                "data_from": inputs.intensity_filtered_data_url,
                "lattice_parameters": lattice_parameter.lattice_parameters,
                "lattice_space_group": lattice_parameter.space_group,
                "idx_for_gen": inputs.rings,
                "idx_for_score": inputs.scoring_rings,
            },
            grains=grains,
            overwrite=inputs.overwrite,
        )
        self.outputs.indexed_grain_data_url = ubi_grains_group_name



from __future__ import annotations

import logging
from typing import Any, Callable

from ewoksorange.bindings import OWEwoksWidgetOneThread
from ewoksorange.bindings.owwidgets import ow_build_opts
from silx.gui import qt

from .utils import format_exception

_logger = logging.getLogger(__name__)


class Ewoks3DXRDWidget(OWEwoksWidgetOneThread, **ow_build_opts):
    want_main_area = True
    want_control_area = False
    resizing_enabled = True

    def __init__(self):
        super().__init__()
        self.setMinimumSize(800, 600)

        self._splitter = qt.QSplitter(qt.Qt.Orientation.Horizontal)
        self._splitter.setSizes([300, 700])
        self.mainArea.layout().addWidget(self._splitter)
        self._input_getters: dict[str, Callable[..., Any]] = {}
        self._input_setters: dict[str, Callable[[Any], None]] = {}

    def registerInput(
        self, name: str, getter: Callable[..., Any], setter: Callable[[Any], None]
    ):
        """Registers an input so its value can be restored at start-up and used at execution"""
        self._input_getters[name] = getter
        self._input_setters[name] = setter

    def _restoreDefaultInputs(self):
        for name, setter in self._input_setters.items():
            try:
                value = self.get_default_input_value(name)
                if value is not None:
                    setter(value)
            except Exception as error:
                _logger.warning(
                    f"Default input restoration failed for {type(self)}.{name}. Cause: {format_exception(error)}"
                )
                self.set_default_input(name, None)
                _logger.warning(f"Default input of {type(self)}.{name} was cleared.")

    def addControlWidget(self, widget: qt.QWidget):
        self._splitter.insertWidget(0, widget)

    def addMainWidget(self, widget: qt.QWidget):
        self._splitter.insertWidget(1, widget)

    def execute_ewoks_task(self, log_missing_inputs=False):
        self._disableControls()
        try:
            self.update_default_inputs(
                **{name: getter() for name, getter in self._input_getters.items()}
            )
            super().execute_ewoks_task(log_missing_inputs)
        except Exception as e:
            self.showError(e)
            self._enableControls()

    def task_output_changed(self):
        super().task_output_changed()
        self._enableControls()
        if self.task_exception is not None:
            self.showError(self.task_exception)
            return

        self.handleSuccessfulExecution()

    def handleSuccessfulExecution(self):
        pass

    def showError(self, error: Exception, title: str | None = None):
        qt.QMessageBox.critical(
            self,
            f"{title if title else self.name} Error",
            format_exception(error),
        )

    def _disableControls(self):
        controlWidget = self._splitter.widget(0)
        if controlWidget:
            controlWidget.setDisabled(True)

    def _enableControls(self):
        controlWidget = self._splitter.widget(0)
        if controlWidget:
            controlWidget.setEnabled(True)
            controlWidget.setEnabled(True)
            controlWidget.setEnabled(True)



from __future__ import annotations

import numpy as np
from silx.gui import qt

from .utils import get_omega_array


class OmegaFrameGroupBox(qt.QGroupBox):
    sigFrameIdxReleased = qt.Signal()

    def __init__(self, parent: qt.QWidget | None = None, **kwargs) -> None:
        super().__init__("Omega and Frame Selection", parent=parent, **kwargs)

        self._omega_array: np.ndarray | None = None
        self._frame_idx: int | None = None

        omega_frame_layout = qt.QGridLayout(self)
        self._frame_slider = qt.QSlider(qt.Qt.Horizontal)
        self._frame_index_label = qt.QLabel("Frame: auto")  # Left side
        self._omega_label = qt.QLabel("Omega: auto")
        omega_frame_layout.addWidget(self._frame_index_label)
        omega_frame_layout.addWidget(self._frame_slider)
        omega_frame_layout.addWidget(self._omega_label)

        # https://doc.qt.io/qt-6/qabstractslider.html#tracking-prop
        self._frame_slider.setTracking(False)
        self._frame_slider.valueChanged.connect(self._on_slider_changed)

    def _on_slider_changed(self, slider_val: int):
        if self._omega_array is None:
            return
        self._frame_idx = slider_val
        omega_value = float(self._omega_array[self._frame_idx])

        self._frame_index_label.setText(f"Frame Index: {self._frame_idx}")
        self._omega_label.setText(f"Omega: {omega_value:.2f}°")

        self.sigFrameIdxReleased.emit()

    def setOmegaArray(self, master_file: str, omega_motor: str, scan_id: str):
        self._omega_array = get_omega_array(
            file_path=master_file,
            omega_motor=omega_motor,
            scan_id=scan_id,
        )
        self._frame_slider.setMinimum(0)
        self._frame_slider.setMaximum(len(self._omega_array) - 1)

    def getFrameIdx(self) -> int | None:
        return self._frame_idx



from __future__ import annotations

from typing import Any

from silx.gui import qt

from ewoks3dxrd.models import SegmenterConfig, SegmenterFolderConfig

from .constants import CorrectionFiles
from .correction_group_box import CorrectionGroupBox
from .folder_metadata_group_box import FolderMetadataGroupBox
from .monitor_segment_group_box import MonitorSegmentGroupBox
from .omega_frame_group_box import OmegaFrameGroupBox
from .segmenter_param_group_box import SegmenterParamGroupBox


class SegmenterSettings(qt.QWidget):
    sigParametersChanged = qt.Signal(dict)

    def __init__(self, parent=None):
        super().__init__(parent)
        self.setWindowTitle("Segmenter Config")
        self.setSizePolicy(qt.QSizePolicy.Expanding, qt.QSizePolicy.Expanding)
        layout = qt.QVBoxLayout()
        self._folder_group = FolderMetadataGroupBox(self)
        layout.addWidget(self._folder_group)
        self._folder_group.sigMasterFileChanged.connect(self._fill_defaults)
        self._omega_frame_group = OmegaFrameGroupBox(self)
        layout.addWidget(self._omega_frame_group)
        self._segmenter_group = SegmenterParamGroupBox(self)
        layout.addWidget(self._segmenter_group)
        self._correction_group = CorrectionGroupBox(self)
        layout.addWidget(self._correction_group)

        self._omega_frame_group.sigFrameIdxReleased.connect(self._on_parameters_changed)
        self._correction_group.sigParamsChanged.connect(self._on_parameters_changed)
        self._segmenter_group.sigParamsChanged.connect(self._on_parameters_changed)

        self._monitor_segment_group = MonitorSegmentGroupBox(self)
        layout.addWidget(self._monitor_segment_group)
        layout.addStretch(1)

        self.setLayout(layout)

    def getParameters(self) -> dict[str, Any]:
        return {
            "segmenter_config": self.getSegmenterConfig(),
            "file_folders": self.getFolderConfig(),
            "correction_files": self.getCorrectionFiles(),
            "monitor_name": self.getMonitorName(),
        }

    def getFolderConfig(self) -> dict[str, Any]:
        return self._folder_group.getConfig().model_dump()

    def setFolderConfig(self, folderConfig: dict):
        self._folder_group.setConfig(SegmenterFolderConfig(**folderConfig))

    def getSegmenterConfig(self) -> dict[str, Any]:
        return self._segmenter_group.getConfig().model_dump()

    def setSegmenterConfig(self, segmenterConfig: dict):
        self._segmenter_group.setConfig(SegmenterConfig(**segmenterConfig))

    def getCorrectionFiles(self) -> CorrectionFiles:
        return self._correction_group.getCorrectionFiles()

    def setCorrectionFiles(self, correctionFiles: dict):
        self._correction_group.setCorrectionFiles(correctionFiles)

    def getMonitorName(self) -> str | None:
        return self._monitor_segment_group.getMonitorName()

    def setMonitorName(self, monitorName: str | None):
        self._monitor_segment_group.setMonitorName(monitorName)

    def _on_parameters_changed(self):
        params = self.getParameters()
        if params:
            self.sigParametersChanged.emit(params)
        else:
            print("Invalid input. Please check the form.")

    def _fill_defaults(self, master_file_path: str):
        motor_name = self._folder_group.getConfig().omega_motor
        self._correction_group.setDefaultCorrectionFiles(motor_name=motor_name)
        scan_number = str(self._folder_group.getScanNumber())
        self._omega_frame_group.setOmegaArray(
            master_file=master_file_path,
            omega_motor=motor_name,
            scan_id=scan_number + ".1",
        )
        self._monitor_segment_group.fillWidgetValues(
            master_file_path=master_file_path, scan_number=scan_number
        )
        self._on_parameters_changed()

    def getScanNumber(self) -> int:
        return self._folder_group.getScanNumber()

    def getFrameIdx(self) -> int | None:
        return self._omega_frame_group.getFrameIdx()

    def getMasterFilePath(self) -> str:
        return self._folder_group.getMasterFilePath()



from pathlib import Path
from typing import Optional

from ewokscore import Task
from pydantic import Field

from ..filtering import filter_by_phase
from ..io import copy_par_file
from ..models import InputsWithOverwrite
from ..nexus.peaks import read_peaks_attributes, save_nexus_process
from ..nexus.utils import (
    check_throw_write_data_group_url,
    get_data_url_paths,
    get_entry_name,
    group_exists,
)


class Inputs(InputsWithOverwrite):
    geometry_updated_data_url: str = Field(
        description="Path to the NXprocess group containing containing the geometry-transformed peaks"
    )
    lattice_file: str = Field(
        description="Path to the `.par` file containing lattice parameters and space group information."
    )
    reciprocal_dist_tol: float = Field(
        description="Tolerance for peak inclusion near lattice rings"
    )
    reciprocal_dist_max: Optional[float] = Field(
        default=None,
        description="Maximum reciprocal distance for filtering. If None or not provided, the maximum value in the `ds` column from input file will be used.",
    )
    process_group_name: str = Field(
        default="lattice_filtered_peaks",
        description="NXprocess group name where to save the filtered peaks",
    )


class FilterByLattice(
    Task,
    input_model=Inputs,
    output_names=["lattice_filtered_data_url", "copied_lattice_file", "ds_tol"],
):
    """
    Performs Lattice/Phase-based filtering on a geometry-transformed 3D peaks file.

    This process applies filtering based on 'reciprocal distance' and 'lattice' rings ds criteria
    to extract relevant peaks.

    ### Steps:
    1. **Initial Filtering:**
        - Copies the input geometry-transformed 3D peaks file.
        - Reads the `ds` column and removes rows where `ds` exceeds the specified `reciprocal_dist_max` value.

    2. **Lattice-Based Filtering:**
        - Computes ideal lattice ring `ds` values (reciprocal distances from the origin).
        - Further filters peaks based on these values, using the tolerance defined by `reciprocal_dist_tol`.

    3. **File Storage:**
        - Saves the lattice-filtered 3D peaks file as
            `{Lattice_name}_{reciprocal_dist_max_tag}_filtered_3d_peaks.h5,flt`
            in the parent directory of the input file ('geometry_trans_3d_peaks_file').

    Additionally, if the specified `lattice_file` is not present in the sample analysis path,
    it is copied to `"par_folder/{lattice_file}"`.

    ### Outputs
    - `lattice_filtered_data_url` (str): Data path to the lattice filtered 'NxProcess group data' peaks
    - `copied_lattice_file` (str): Path to the copied lattice parameter file stored within the analysis folder.
    - `ds_tol` (float): Forwarding the value of `reciprocal_dist_tol` for the future tasks
    """

    def run(self):
        inputs = Inputs(**self.get_input_values())

        nexus_file_path, geometry_updated_data_group_path = get_data_url_paths(
            inputs.geometry_updated_data_url
        )
        if not group_exists(nexus_file_path, geometry_updated_data_group_path):
            raise FileNotFoundError(
                f""" File or Data Group not Found Error,
                Either it is missing nexus file path: {nexus_file_path} or
                geometry updated data group path: {geometry_updated_data_group_path}.
                """
            )

        entry_name = get_entry_name(geometry_updated_data_group_path)
        check_throw_write_data_group_url(
            inputs.overwrite,
            nexus_file_path,
            f"{entry_name}/{inputs.process_group_name}",
        )

        lattice_file = copy_par_file(inputs.lattice_file, Path(nexus_file_path).parent)

        peaks = read_peaks_attributes(
            filename=nexus_file_path,
            process_group=geometry_updated_data_group_path,
        )

        ds_tol = inputs.reciprocal_dist_tol
        ds_max = inputs.reciprocal_dist_max
        filtered_peaks_cf, unit_cell_parameters = filter_by_phase(
            peaks,
            lattice_file=lattice_file,
            ds_tol=ds_tol,
            ds_max=ds_max,
        )

        nxprocess_url = save_nexus_process(
            filename=nexus_file_path,
            entry_name=entry_name,
            process_name=inputs.process_group_name,
            peaks_data=filtered_peaks_cf,
            config_settings={
                "lattice_name": lattice_file.stem,
                "lattice_parameters": unit_cell_parameters.lattice_parameters,
                "lattice_space_group": unit_cell_parameters.space_group,
                "dsmax": ds_max if ds_max is None else "None",
                "dstol": ds_tol,
                "data_from": f"{nexus_file_path}::{geometry_updated_data_group_path}",
            },
            pks_axes=("ds", "eta"),
            overwrite=inputs.overwrite,
        )

        self.outputs.lattice_filtered_data_url = nxprocess_url
        self.outputs.copied_lattice_file = str(lattice_file)
        self.outputs.ds_tol = ds_tol



from __future__ import annotations

from ewokscore import missing_data
from silx.gui import qt


class ProcessNameLineEdit(qt.QLineEdit):

    def __init__(self, default: str, parent: qt.QWidget | None = None):
        super().__init__(parent)
        self.setPlaceholderText(default)
        self.label = "Output NeXus Group Name"
        self.setToolTip(
            f"Name of the NeXus group where data will be saved. Default: '{default}'"
        )

    def getText(self) -> str | missing_data.MissingData:
        value = self.text().strip()
        if value == "":
            return missing_data.MISSING_DATA
        return value



from __future__ import annotations

from enum import Enum


class Symmetry(str, Enum):
    cubic = "cubic"
    hexagonal = "hexagonal"
    trigonal = "trigonal"
    rhombohedralP = "rhombohedralP"
    trigonalP = "trigonalP"
    tetragonal = "tetragonal"
    orthorhombic = "orthorhombic"
    monoclinic_c = "monoclinic_c"
    monoclinic_a = "monoclinic_a"
    monoclinic_b = "monoclinic_b"
    triclinic = "triclinic"



from __future__ import annotations
from enum import Enum
from typing import Dict, Optional, Tuple, TypedDict, Union


class OmegaMotor(str, Enum):
    diffrz = "diffrz"
    omega = "omega"


class Detector(str, Enum):
    frelon1 = "frelon1"
    frelon3 = "frelon3"
    eiger = "eiger"


class CorrectionFiles(TypedDict):
    bg_file: Optional[str]
    mask_file: Optional[str]
    dark_file: Optional[str]
    flat_file: Optional[str]


SEGMENTER_DEFAULTS: Dict[str, Union[float, int]] = {
    "threshold": 70,
    "smooth_sigma": 1.0,
    "bgc": 0.9,
    "min_px": 3,
    "offset_threshold": 100,
    "ratio_threshold": 150,
}

DEFAULTS_CORRECTION_MAP: Dict[str, CorrectionFiles] = {
    "diffrz": {
        "bg_file": None,
        "mask_file": "/data/projects/id03_3dxrd/expt/PROCESSED_DATA/mask.edf",
        "dark_file": None,
        "flat_file": None,
    },
    "omega": {
        "bg_file": "/data/projects/id03_3dxrd/id03_expt/PROCESSED_DATA/bkg_avg.edf",
        "mask_file": "/data/projects/id03_3dxrd/id03_expt/PROCESSED_DATA/frelon1_mask_20250122.edf",
        "dark_file": None,
        "flat_file": None,
    },
}

MONITOR_KEYS: Tuple[str, str] = ("pico", "fpico")

SEGMENTER_TOOLTIPS: Dict[str, str] = {
    "bg_file": """
        *Optional: File containing detector background image.
        """,
    "mask_file": """
        *Optional: File describing the detector mask.
        """,
    "flat_file": """
        *Optional: File containing detector sensitivity image.
        """,
    "dark_file": """
        *Optional: File containing detector offset image.
        """,
    "threshold": """
        Minimum pixel intensity (in ADU) to be considered a potential peak.
        \tUsed to eliminate low-signal noise.
        \tTypical range: 50-?
        """,
    "smooth_sigma": """
        Gaussian blur sigma value used for background smoothing.
        \tHigher values result in more smoothing.
        \tTypical range: 0.5-2.0
        """,
    "bgc": """
        Fractional background intensity value (in ADU).
        \tFractional part of background per peak to remove.
        \tTypical range: 0.7-1.0
        """,
    "min_px": """
        Minimum number of connected pixels required to consider a region as a peak.
        \tTypical range: 1-?
        """,
    "offset_threshold": """
        Set intensity to a constant if it is less than this value.
        \tTypical range: 100-?
        \tShould satisfy: offset_threshold < ratio_threshold.
        """,
    "ratio_threshold": """
        Used to filter out peaks with an intensity higher than this value.
        \tTypical range: 150-?
        \tShould satisfy: offset_threshold < ratio_threshold.
        """,
}



import os

# Because of OpenMP and Multiprocessing mix, we are suppressing OpenMP usage in the C code of cImageD11
# See https://github.com/FABLE-3DXRD/ImageD11/issues/486 and https://gitlab.esrf.fr/workflow/ewoksapps/ewoks3dxrd/-/issues/19
os.environ["OMP_NUM_THREADS"] = "1"



from __future__ import annotations
from silx.gui import qt


class FilenameCompleterLineEdit(qt.QLineEdit):
    def __init__(self, parent: qt.QWidget | None = None, **kwargs) -> None:
        super().__init__(parent=parent, **kwargs)

        completer = qt.QCompleter()
        model = qt.QFileSystemModel(completer)
        model.setOption(qt.QFileSystemModel.Option.DontWatchForChanges, True)
        model.setRootPath(qt.QDir.rootPath())

        completer.setModel(model)
        completer.setCompletionRole(qt.QFileSystemModel.Roles.FileNameRole)
        self.setCompleter(completer)
        self.installEventFilter(self)

    def eventFilter(self, source, event):
        if event.type() == qt.QtCore.QEvent.Type.ToolTip and self.text():
            qt.QToolTip.showText(event.globalPos(), self.text(), self)
            return True
        return super().eventFilter(source, event)



from __future__ import annotations

import numpy as np
import qtawesome
from silx.gui import qt
from silx.gui.colors import Colormap
from silx.gui.dialog.ColormapDialog import ColormapDialog
from silx.gui.plot import PlotWidget, actions
from silx.gui.plot.items import Scatter
from silx.gui.plot.tools import PositionInfo

_UNFILTERED_PEAKS_LEGEND = "unfiltered"


class _PeakFilterToolBar(qt.QToolBar):
    togglePeaks = qt.Signal(bool)

    def __init__(
        self,
        plot: PlotWidget,
        parent=None,
    ):
        super().__init__("Plot Tools", parent)

        self.setToolButtonStyle(qt.Qt.ToolButtonStyle.ToolButtonTextBesideIcon)
        zoomAction = actions.mode.ZoomModeAction(parent=self, plot=plot)
        panAction = actions.mode.PanModeAction(parent=self, plot=plot)
        self.addAction(panAction)
        self.addAction(zoomAction)
        plot.setInteractiveMode("pan")

        resetZoom = actions.control.ResetZoomAction(parent=self, plot=plot)
        self.addAction(resetZoom)
        colormapAction = actions.control.ColormapAction(parent=self, plot=plot)
        colormapDialog = ColormapDialog(parent=self)
        colormapAction.setColorDialog(colormapDialog)
        self.addAction(colormapAction)
        self._togglePeaksAction = qt.QAction(
            qtawesome.icon("fa6s.xmark", rotated=45), "Show/Hide Unfiltered Peaks", self
        )
        self._togglePeaksAction.setIconText("Unfiltered peaks")
        self._togglePeaksAction.setCheckable(True)
        self._togglePeaksAction.setChecked(True)
        self._togglePeaksAction.triggered.connect(self.togglePeaks)
        self.addAction(self._togglePeaksAction)


class PeakFilterPlot2D(qt.QMainWindow):

    def __init__(self, parent=None):
        super().__init__(parent)
        self._initCentralLayout()
        toolbar = _PeakFilterToolBar(
            plot=self._plot,
            parent=self,
        )
        toolbar.togglePeaks.connect(self._toggleInPeaks)
        toolbar.setToolButtonStyle(qt.Qt.ToolButtonStyle.ToolButtonTextBesideIcon)
        self.addToolBar(qt.Qt.TopToolBarArea, toolbar)
        self._toolbar = toolbar

    def _initCentralLayout(self):
        centralWidget = qt.QWidget(self)
        self.setCentralWidget(centralWidget)

        self._plot = PlotWidget(parent=centralWidget, backend="gl")
        self._plot.setGraphGrid(True)
        self._plot.setAxesMargins(0.05, 0.03, 0.03, 0.05)
        self._plot.setKeepDataAspectRatio(False)
        positionInfo = PositionInfo(plot=self._plot, parent=centralWidget)

        layout = qt.QGridLayout(centralWidget)
        layout.setSpacing(0)
        layout.setContentsMargins(0, 0, 0, 0)
        layout.addWidget(self._plot, 0, 0)
        layout.addWidget(positionInfo, 1, 0, 1, 2)

        centralWidget.setLayout(layout)

    def getPlotWidget(self) -> PlotWidget:
        return self._plot

    def addUnfilteredScatter(
        self,
        x: np.ndarray,
        y: np.ndarray,
        value: np.ndarray,
        colormap: Colormap | dict,
    ) -> Scatter:
        scatter = self._plot.addScatter(
            x=x,
            y=y,
            value=value,
            colormap=colormap,
            symbol="+",
            legend=_UNFILTERED_PEAKS_LEGEND,
        )
        scatter.setSymbolSize(7)
        return scatter

    def addFilteredScatter(
        self,
        x: np.ndarray,
        y: np.ndarray,
        value: np.ndarray,
        colormap: Colormap | dict,
    ) -> Scatter:
        scatter = self._plot.addScatter(
            x=x,
            y=y,
            value=value,
            colormap=colormap,
            symbol="o",
            legend="filtered",
        )
        scatter.setSymbolSize(7)
        return scatter

    def resetZoom(self):
        self._plot.resetZoom()

    def setYAxisLogarithmic(self, flag=True):
        self._plot.setYAxisLogarithmic(flag)

    def setGraphXLabel(self, str_val: str):
        self._plot.setGraphXLabel(str_val)

    def setGraphYLabel(self, label: str):
        self._plot.setGraphYLabel(label)

    def _toggleInPeaks(self, checked: bool):
        scatter = self._plot.getScatter(legend=_UNFILTERED_PEAKS_LEGEND)
        if scatter:
            scatter.setVisible(checked)

    def addCurve(self, x: np.ndarray, y: np.ndarray, legend: str, linewidth: int):
        self._plot.addCurve(x, y, legend=legend, linewidth=linewidth)



from typing import Sequence

from ewokscore import Task
from ImageD11.unitcell import unitcell as UnitCell
from pydantic import Field

from ..filtering import filter_with_indexer
from ..models import InputsWithOverwrite
from ..nexus.peaks import read_peaks_attributes, save_nexus_process
from ..io import read_lattice_cell_data
from ..nexus.utils import (
    check_throw_write_data_group_url,
    get_data_url_paths,
    get_entry_name,
)


class Inputs(InputsWithOverwrite):
    intensity_filtered_data_url: str = Field(
        description="Path to the NXprocess group containing the intensity filtered peaks"
    )
    rings: Sequence[int] = Field(
        description="Ring indices",
        examples=[(0, 1, 2)],
    )
    process_group_name: str = Field(
        default="indexer_filtered_peaks",
        description="NXprocess group name where to save the filtered peaks",
    )
    wavelength: float = Field(
        description="wavelength used in the geometry transformation task",
    )
    ds_tol: float = Field(
        default=0.025,
        description="distance tolerance used in the lattice based filtering task",
    )
    lattice_file: str = Field(
        description="Path to the file (`.par` or `.cif`) containing lattice parameters and space group information forwarded by lattice filter task"
    )


class FilterByIndexer(
    Task,
    input_model=Inputs,
    output_names=["indexer_filtered_data_url"],
):
    """
    Filter the 3D merged peaks using Indexer, to be used only to grid indexer for producing grains.

    Outputs:

    - `indexer_filtered_data_url` (str): Data path to 'NxProcess group data' Grains that stores generated UBI

    """

    def run(self):
        inputs = Inputs(**self.get_input_values())
        nexus_file_path, intensity_filtered_data_group_path = get_data_url_paths(
            inputs.intensity_filtered_data_url
        )

        entry_name = get_entry_name(intensity_filtered_data_group_path)
        output_process_path = f"{entry_name}/{inputs.process_group_name}"
        check_throw_write_data_group_url(
            overwrite=inputs.overwrite,
            filename=nexus_file_path,
            data_group_path=output_process_path,
        )

        intensity_filtered_peaks = read_peaks_attributes(
            filename=nexus_file_path, process_group=intensity_filtered_data_group_path
        )
        lattice_parameter = read_lattice_cell_data(inputs.lattice_file)
        unit_cell = UnitCell(
            lattice_parameters=lattice_parameter.lattice_parameters,
            symmetry=str(lattice_parameter.space_group),
        )

        index_filtered_peaks = filter_with_indexer(
            peak_3d_dict=intensity_filtered_peaks,
            unit_cell=unit_cell,
            wavelength=inputs.wavelength,
            phase_ds_tolerance=inputs.ds_tol,
            rings=inputs.rings,
        )

        nxprocess_url = save_nexus_process(
            filename=nexus_file_path,
            entry_name=entry_name,
            process_name=inputs.process_group_name,
            peaks_data=index_filtered_peaks,
            config_settings={
                "dstol": inputs.ds_tol,
                "rings": inputs.rings,
                "data_from": inputs.intensity_filtered_data_url,
            },
            pks_axes=("ds", "eta"),
            overwrite=inputs.overwrite,
        )
        self.outputs.indexer_filtered_data_url = nxprocess_url



from __future__ import annotations
from typing import Any
import h5py
from pathlib import Path
from typing import Tuple
import numpy as np
from silx.io.utils import h5py_read_dataset
from .utils import get_data_url_paths


def _save_parameters(
    parent_group: h5py.Group,
    config_settings: dict[str, Any],
    group_name: str = "parameters",
):
    """
    Helper function to place the config_settings in 'param_name' group
    in the given `parent_group`
    """
    parameters_group = parent_group.require_group(group_name)
    parameters_group.attrs["NX_class"] = "NXcollection"

    for key, value in config_settings.items():
        if key in parameters_group:
            del parameters_group[key]
        parameters_group.create_dataset(
            key, data=value if value is not None else "None"
        )


def create_parameters_group(
    group_path: h5py.Group,
    config_settings: dict[str, Any] | dict[str, dict[str, Any]],
):
    if isinstance(config_settings, dict) and all(
        isinstance(value, dict) for value in config_settings.values()
    ):
        parameters_group = group_path.require_group("parameters")
        parameters_group.attrs["NX_class"] = "NXcollection"
        for param_name, param_values in config_settings.items():
            _save_parameters(
                parent_group=parameters_group,
                config_settings=param_values,
                group_name=param_name,
            )
    else:
        _save_parameters(parent_group=group_path, config_settings=config_settings)


def get_parameters(filename: str | Path, process_group_name: str) -> dict[str, str]:
    return get_parameter(f"{filename}::{process_group_name}", key=None)


def get_parameter(data_url_as_str: str, key: str | None, sub_folder="parameters"):
    file_path, data_path = get_data_url_paths(data_url_as_str)

    with h5py.File(file_path, "r") as h5file:
        parameters: h5py.Group = h5file[f"{data_path}/{sub_folder}"]
        if key is None:
            return {k: h5py_read_dataset(dset=v) for k, v in parameters.items()}

        if key not in parameters:
            return None

        return h5py_read_dataset(h5file[f"{data_path}/{sub_folder}/{key}"])


def find_parameter(data_url_as_str: str, key: str):
    value = get_parameter(data_url_as_str, key=key)
    if value is not None:
        return value

    data_from = get_parameter(data_url_as_str, key="data_from")
    if data_from is None:
        raise KeyError(f"Could not find {key} nor 'data_from' in {data_url_as_str}")

    data_from_value = find_parameter(data_from, key=key)
    if data_from_value is None:
        raise KeyError(f"Could not find {key} in {data_from}/parameters.")

    return data_from_value


def read_nx_collection_parameters(
    filename: str | Path, process_group_name: str, sub_folder: str
) -> dict:
    return get_parameter(
        data_url_as_str=f"{filename}::{process_group_name}",
        key=None,
        sub_folder=sub_folder,
    )


def get_omega_array(
    filename: str | Path, entry_name: str, process_group_name: str
) -> np.ndarray:
    with h5py.File(filename, "r") as f:
        entry = f[f"{entry_name}/{process_group_name}"]
        folder_grp = entry["parameters/FolderFileSettings"]
        masterfile = h5py_read_dataset(dset=folder_grp["masterfile"])
        scan_number = h5py_read_dataset(dset=folder_grp["scan_number"])
        omegamotor = h5py_read_dataset(dset=folder_grp["omegamotor"])
        with h5py.File(masterfile, "r") as hin:
            omega_angles = hin[f"{scan_number}.1/measurement"][omegamotor]
            return h5py_read_dataset(dset=omega_angles)


def find_lattice_parameters(data_url_as_str: str) -> Tuple[np.ndarray, int]:
    lattice_params = find_parameter(data_url_as_str, key="lattice_parameters")
    symmetry = find_parameter(data_url_as_str, key="lattice_space_group")

    return lattice_params, int(symmetry)


def find_wavelength(data_url_as_str: str) -> float:
    return np.float64(find_parameter(data_url_as_str, key="wavelength"))


def find_reciprocal_distance_tolerance(data_url_as_str: str) -> float:
    return np.float64(find_parameter(data_url_as_str, key="dstol"))


def find_lattice_nexus_group_url(data_url_as_str: str):
    file_path, data_path = get_data_url_paths(data_url_as_str)
    with h5py.File(file_path, "r") as f:
        par_grp = f[f"{data_path}/parameters"]
        if "lattice_name" in par_grp:
            return f"{file_path}::{data_path}"
        elif "data_from" in par_grp:
            return find_lattice_nexus_group_url(
                h5py_read_dataset(dset=par_grp["data_from"])
            )
        else:
            raise FileNotFoundError(
                """Could not find phase nexus data group in the given data_url path.
                """
            )


def get_lattice_parameters(
    filename: str | Path, entry_name: str, process_group_name: str
) -> Tuple[np.ndarray, int]:
    return find_lattice_parameters(f"{filename}::/{entry_name}/{process_group_name}")


def get_wavelength(
    filename: str | Path, entry_name: str, process_group_name: str
) -> float:
    return find_wavelength(f"{filename}::/{entry_name}/{process_group_name}")



from __future__ import annotations
from silx.gui import qt


class DebounceTimer(qt.QTimer):
    def __init__(
        self, callback: callable[[], None], timeout_ms: int = 200, parent=None
    ):
        super().__init__(parent)
        self._callback = callback
        self.setSingleShot(True)
        self.setInterval(timeout_ms)
        self.timeout.connect(self._callback)



from __future__ import annotations
from typing import Any, Optional, Tuple
import numpy as np
import h5py
from pathlib import Path
from ImageD11 import columnfile as PeakColumnFile
from .parameters import create_parameters_group, get_data_url_paths


def create_nexus_peaks_data_group(
    group_path: h5py.Group,
    peaks_data: dict[str, np.ndarray],
    pks_axes: Tuple[str, str],
    signal_name: str,
    scale: str,
    soft_ln_pks_path: Optional[str] = None,
) -> str:

    peak_data_group = group_path.require_group("peaks")
    peak_data_group.attrs["NX_class"] = "NXdata"
    for key, value in peaks_data.items():
        peak_data_group.create_dataset(key, data=value)

    if soft_ln_pks_path:
        soft_ln_pks_group = group_path.file[soft_ln_pks_path]
        if "peaks" in soft_ln_pks_group:
            soft_link_peaks = soft_ln_pks_group["peaks"]
            for key in soft_link_peaks.keys():
                if key not in peak_data_group:
                    peak_data_group[key] = h5py.SoftLink(soft_link_peaks[key].name)

    peak_data_group.attrs["axes"] = pks_axes
    peak_data_group.attrs["signal"] = signal_name

    if scale not in ["log", "linear"]:
        raise ValueError("scale must be 'log' or 'linear'")

    peak_data_group.attrs["scale"] = scale
    return peak_data_group.name


def save_nexus_process(
    filename: str | Path,
    entry_name: str,
    process_name: str,
    peaks_data: dict[str, np.ndarray],
    config_settings: dict[str, Any] | dict[str, dict[str, Any]],
    pks_axes: Tuple[str, str] = ("f_raw", "s_raw"),
    signal_name: str = "Number_of_pixels",
    scale: str = "log",
    overwrite: bool = False,
    ln_pks_from_group_name: Optional[str] = None,
) -> str:

    with h5py.File(filename, "a") as h5_file:
        entry = h5_file.require_group(entry_name)
        entry.attrs["NX_class"] = "NXentry"

        if process_name in entry:
            if overwrite:
                del entry[process_name]
            else:
                raise FileExistsError(
                    f"""IN the nexus process file name {filename}, there is already a
                    nexus process group {process_name} exists.
                    To overwrite provide a overwrite permission.
                    """
                )

        process_group = entry.create_group(process_name)
        process_group.attrs["NX_class"] = "NXprocess"
        create_nexus_peaks_data_group(
            group_path=process_group,
            peaks_data=peaks_data,
            soft_ln_pks_path=ln_pks_from_group_name,
            pks_axes=pks_axes,
            signal_name=signal_name,
            scale=scale,
        )
        create_parameters_group(
            group_path=process_group,
            config_settings=config_settings,
        )
        return f"{filename}::{process_group.name}"


def read_peaks_attributes(
    filename: str | Path, process_group: str
) -> dict[str, np.ndarray]:
    """
    Extract peaks column data stored in {entry_name}/{process_name}/peaks
    Inputs:
        filename: file path to .h5 file
        entry_name: entry point inside .h5 file
        process_name: group name inside the entry point
    """
    with h5py.File(filename, "r") as f:
        peaks_group = f[f"{process_group}/peaks"]
        return {name: dataset[()] for name, dataset in peaks_group.items()}


def save_column_file_as_ascii(input_data_url: str, output_file_name: Path) -> Path:
    cf_file_path, cf_group_path = get_data_url_paths(input_data_url)
    peaks_dict = read_peaks_attributes(
        filename=cf_file_path, process_group=cf_group_path
    )
    cf = PeakColumnFile.colfile_from_dict(peaks_dict)

    cf.writefile(output_file_name)

    return output_file_name



import h5py
from silx.io.url import DataUrl

from ewoks3dxrd.tasks.geometry_transformation import GeometryTransformation


GEOMETRY = {
    "chi": 0,
    "distance": 277938.9537523063,
    "fit_tolerance": 0.05,
    "min_bin_prob": 1e-05,
    "no_bins": 10000,
    "o11": 1,
    "o12": 0,
    "o21": 0,
    "o22": -1,
    "omegasign": 1.0,
    "t_x": 0.0,
    "t_y": 0.0,
    "t_z": 0.0,
    "tilt_x": -0.003145870872293617,
    "tilt_y": -0.0019304888034885003,
    "tilt_z": 0.00924114210818807,
    "wavelength": 0.22477970523350738,
    "wedge": 0.10331991684510052,
    "weight_hist_intensities": 0,
    "y_center": 1009.4810919042234,
    "y_size": 47.3,
    "z_center": 1009.4248984777241,
    "z_size": 47.3,
}


def test_execution(nexus_3dxrd_file, tmp_path):
    geom_file = tmp_path / "geometry_closer.par"

    with open(geom_file, "w") as f:
        for key, value in GEOMETRY.items():
            f.write(f"{key} {value}\n")

    inputs = {
        "spatial_corrected_data_url": f"{nexus_3dxrd_file}::/1.1/geometry_updated_peaks",
        "geometry_par_file": "/data/projects/id03_3dxrd/id03_expt/PROCESSED_DATA/geometry_closer.par",
        "overwrite": True,
    }

    task = GeometryTransformation(inputs=inputs)
    task.execute()
    output_url = DataUrl(task.outputs["geometry_updated_data_url"])

    with h5py.File(output_url.file_path(), "r") as h5file:
        process_group = h5file[output_url.data_path()]
        assert isinstance(process_group, h5py.Group)
        assert isinstance(process_group["peaks/ds"], h5py.Dataset)
        for name, dset in process_group["parameters"].items():
            assert dset[()] == GEOMETRY[name]



from __future__ import annotations

from silx.gui import qt


class CollapsibleWidget(qt.QWidget):

    def __init__(self, title: str = "", parent: qt.QWidget | None = None) -> None:
        super().__init__(parent=parent)
        layout = qt.QVBoxLayout(self)
        layout.setContentsMargins(0, 0, 0, 0)

        self._toolButton = qt.QToolButton()
        self._toolButton.setSizePolicy(
            qt.QSizePolicy.Policy.Expanding, qt.QSizePolicy.Policy.Fixed
        )
        self._toolButton.setText(title)
        self._toolButton.setToolButtonStyle(
            qt.Qt.ToolButtonStyle.ToolButtonTextBesideIcon
        )
        self._toolButton.setCheckable(True)
        self._toolButton.toggled.connect(self._toggleIcon)
        self._toolButton.setChecked(True)
        layout.addWidget(self._toolButton)

        self._contentsFrame = qt.QGroupBox()
        layout.addWidget(self._contentsFrame)
        self._toolButton.toggled.connect(self._contentsFrame.setVisible)

    def setLayout(self, layout: qt.QLayout | None) -> None:
        self._contentsFrame.setLayout(layout)

    def _toggleIcon(self, toggled: bool) -> None:
        self._toolButton.setArrowType(
            qt.Qt.ArrowType.DownArrow if toggled else qt.Qt.ArrowType.RightArrow
        )

    def isCollapsed(self) -> bool:
        return not self._toolButton.isChecked()

    def setCollapsed(self, collapsed: bool):
        self._toolButton.setChecked(not collapsed)



from __future__ import annotations

from typing import Tuple

from silx.gui import qt

from ..common.double_spinbox_range_widget import DoubleSpinBoxRangeWidget
from ..common.tolerance_selector import ToleranceSelector
from ..indexer.constants import Symmetry


class GrainMapParameterGroupBox(qt.QGroupBox):
    def __init__(self, parent: qt.QWidget | None = None, **kwargs) -> None:
        super().__init__("Refinement Settings", parent=parent, **kwargs)
        layout = qt.QFormLayout(self)

        self._minpks = qt.QSpinBox()
        self._minpks.setRange(0, 10000)
        self._minpks.setValue(1)
        self._minpks.setToolTip(
            "Minimal number of peaks for the grain to be kept after iterative refinement."
        )

        self._intensityTwoThetaRange = DoubleSpinBoxRangeWidget(default=(0.0, 180.0))
        self._intensityTwoThetaRange.setToolTip(
            "Range of two theta to use when refining. Only peaks in this range will be selected"
        )

        self._symmetry = qt.QComboBox()
        self._symmetry.addItems([e.value for e in Symmetry])
        self._symmetry.setToolTip("Lattice symmetry used to further refine grains")
        self._toleranceSelector = ToleranceSelector()

        layout.addRow("Min Peaks for each grain", self._minpks)
        layout.addRow("2θ range", self._intensityTwoThetaRange)
        layout.addRow("Crystal Symmetry", self._symmetry)
        layout.addRow("Tolerance Level", self._toleranceSelector)

    def getSymmetry(self) -> str:
        return self._symmetry.currentText()

    def getTwoThetaRange(self) -> tuple[float, float]:
        return self._intensityTwoThetaRange.getValue()

    def getMinPeaks(self) -> int:
        return self._minpks.value()

    def getTolerances(self) -> Tuple[float, ...]:
        return self._toleranceSelector.getValue()

    def setSymmetry(self, value: str):
        return self._symmetry.setCurrentText(value)

    def setTwoThetaRange(self, value: tuple[float, float]):
        return self._intensityTwoThetaRange.setValue(value)

    def setMinPeaks(self, value: int):
        return self._minpks.setValue(value)

    def setTolerances(self, value: Tuple[float, ...]):
        return self._toleranceSelector.setValue(value)



from __future__ import annotations
from silx.gui import qt
from .filename_completer_line_edit import FilenameCompleterLineEdit


class FileFolderBrowseButton(qt.QWidget):
    def __init__(
        self,
        parent: qt.QWidget | None = None,
        dialogTitle: str = "",
        directory: bool = False,
        **kwargs,
    ) -> None:
        super().__init__(parent=parent, **kwargs)

        layout = qt.QHBoxLayout(self)
        layout.setContentsMargins(0, 0, 0, 0)

        self._line_edit = FilenameCompleterLineEdit(self)
        _browse_btn = qt.QPushButton("Browse...")

        def open_dialog():
            if directory:
                path = qt.QFileDialog.getExistingDirectory(self, dialogTitle)
            else:
                path, _ = qt.QFileDialog.getOpenFileName(self, dialogTitle)
            if path:
                self._line_edit.setText(path)

        _browse_btn.clicked.connect(open_dialog)

        layout.addWidget(self._line_edit)
        layout.addWidget(_browse_btn)

    def getText(self) -> str:
        return self._line_edit.text().strip()

    def setText(self, path: str):
        self._line_edit.setText(path)

    def eventFilter(self, source, event):
        if event.type() == qt.QtCore.QEvent.Type.ToolTip and self.getText():
            qt.QToolTip.showText(event.globalPos(), self.getText(), self)
            return True
        return super().eventFilter(source, event)

    def clearText(self):
        self._line_edit.clear()



from __future__ import annotations
import weakref
from enum import StrEnum

from silx.gui import qt
from silx.gui.colors import cursorColorForColormap
from silx.gui.plot import PlotWidget


class ColorOption(StrEnum):
    AUTO = "Auto"
    BLACK = "Black"
    WHITE = "White"


class PeakColorComboBox(qt.QComboBox):
    currentColorChanged = qt.Signal(str)

    def __init__(self, plot: PlotWidget, parent=None):
        super().__init__(parent)
        self._plotRef = weakref.ref(plot)
        for option in ColorOption:
            self.addItem(option.value)

        self.currentTextChanged.connect(self._emitNewColor)

    def _getPlotColormapName(self) -> str | None:
        plot = self._plotRef()
        if plot is None:
            return None

        return plot.getDefaultColormap().getName()

    def _optionToColor(self, option: str) -> str:
        if option == ColorOption.BLACK:
            return "black"
        elif option == ColorOption.WHITE:
            return "white"
        elif option == ColorOption.AUTO:
            currentColormapName = self._getPlotColormapName()
            if currentColormapName is None:
                return "pink"
            return cursorColorForColormap(currentColormapName)
        else:
            raise ValueError(f"Unknown option: {option}")

    def _emitNewColor(self, option: str):
        color = self._optionToColor(option)
        self.currentColorChanged.emit(color)

    def getCurrentColor(self) -> str:
        return self._optionToColor(self.currentText())



from __future__ import annotations

from typing import Optional

from ewokscore import missing_data
from silx.gui import qt
from silx.gui.plot3d.SceneWindow import SceneWindow

from ewoks3dxrd.tasks.make_grain_map import MakeGrainMap

from .common.ewoks3dxrd_widget import Ewoks3DXRDWidget
from .common.grain_size_slider import GrainSizeSlider
from .common.sphere import Spheres, build_grain_spheres
from .makegrains.grain_map_parameter_group_box import GrainMapParameterGroupBox


class OWMakeGrainMap(Ewoks3DXRDWidget, ewokstaskclass=MakeGrainMap):
    name = "Refine Grain Mapping"
    description = "Second Stage Indexing (refine grain positions)."
    icon = "icons/maps.svg"
    _ewoks_inputs_to_hide_from_orange = (
        "overwrite",
        "intensity_frac",
        "hkl_tols",
        "minpks",
        "intensity_two_theta_range",
        "symmetry",
        "analyse_folder",
        "process_group_name",
    )

    def __init__(self):
        super().__init__()
        self._settingsPanelWidget = qt.QWidget(self)
        settingLayout = qt.QVBoxLayout(self._settingsPanelWidget)

        self._settingsPanel = GrainMapParameterGroupBox(self)
        settingLayout.addWidget(self._settingsPanel)

        executeOverWriteLayout = qt.QFormLayout()
        self._inputGrainDataUrl = qt.QLineEdit()
        self._inputStrongFilteredPeaksDataUrl = qt.QLineEdit()

        self._allAlignedPeaksWidget = qt.QWidget()
        allAlignedPeaksLayout = qt.QHBoxLayout(self._allAlignedPeaksWidget)
        allAlignedPeaksLayout.setContentsMargins(0, 0, 0, 0)
        self._allAlignedPeaksDataUrl = qt.QLineEdit()
        self._allAlignedPeaksCheckbox = qt.QCheckBox("Use all Peaks")
        allAlignedPeaksLayout.addWidget(self._allAlignedPeaksDataUrl)
        allAlignedPeaksLayout.addWidget(self._allAlignedPeaksCheckbox)
        self._allAlignedPeaksWidget.setToolTip(
            "Use all peaks aligned with lattice rings to validate grains. Otherwise, grains are refined using only strongly filtered peaks."
        )

        self._latticeParFile = qt.QLineEdit()
        self._overwrite = qt.QCheckBox()

        self._refineGrainsBtn = qt.QPushButton("Refine Grains")
        executeOverWriteLayout.addRow("Input Grain Data URL", self._inputGrainDataUrl)
        executeOverWriteLayout.addRow(
            "Strong Filtered Peaks URL", self._inputStrongFilteredPeaksDataUrl
        )
        executeOverWriteLayout.addRow("All Peaks Data URL", self._allAlignedPeaksWidget)
        executeOverWriteLayout.addRow(
            "Lattice Parameter File Path", self._latticeParFile
        )
        executeOverWriteLayout.addRow("Overwrite", self._overwrite)
        executeOverWriteLayout.addRow(self._refineGrainsBtn)

        self._grainSlider = GrainSizeSlider()
        self._grainSlider.floatValueChanged.connect(self._updateGrainSize)
        self._grainSphere: Optional[Spheres] = None
        executeOverWriteLayout.addRow("Grain Size", self._grainSlider)

        settingLayout.addLayout(executeOverWriteLayout)
        self.addControlWidget(self._settingsPanelWidget)

        self._refineGrainsPlot = SceneWindow(self)
        self._refineGrainsPlot.getParamTreeView().parent().setVisible(False)
        self._refineGrainsPlot.getGroupResetWidget().parent().setVisible(False)
        self._refineGrainsPlot.getSceneWidget().setBackgroundColor((0.8, 0.8, 0.8, 1.0))
        self._refineGrainsPlot.setSizePolicy(
            qt.QSizePolicy.Expanding, qt.QSizePolicy.Expanding
        )
        self.addMainWidget(self._refineGrainsPlot)

        self._refineGrainsBtn.clicked.connect(self.execute_ewoks_task)

        self.registerInput(
            "indexed_grain_data_url",
            self._inputGrainDataUrl.text,
            self._inputGrainDataUrl.setText,
        )
        self.registerInput(
            "intensity_filtered_data_url",
            self._inputStrongFilteredPeaksDataUrl.text,
            self._inputStrongFilteredPeaksDataUrl.setText,
        )
        self.registerInput(
            "hkl_tols",
            self._settingsPanel.getTolerances,
            self._settingsPanel.setTolerances,
        )
        self.registerInput(
            "minpks", self._settingsPanel.getMinPeaks, self._settingsPanel.setMinPeaks
        )
        self.registerInput(
            "intensity_fine_filtered_data_url",
            self._getIntensityFineFilteredDataUrl,
            self._allAlignedPeaksDataUrl.setText,
        )
        self.registerInput(
            "intensity_two_theta_range",
            self._settingsPanel.getTwoThetaRange,
            self._settingsPanel.setTwoThetaRange,
        )
        self.registerInput(
            "symmetry", self._settingsPanel.getSymmetry, self._settingsPanel.setSymmetry
        )
        self.registerInput(
            "lattice_file", self._latticeParFile.text, self._latticeParFile.setText
        )
        self.registerInput(
            "overwrite", self._overwrite.isChecked, self._overwrite.setChecked
        )
        self._restoreDefaultInputs()

    def _getIntensityFineFilteredDataUrl(self) -> missing_data.MissingData | str:
        url = self._allAlignedPeaksDataUrl.text().strip()
        if not url:
            return missing_data.MISSING_DATA

        return url

    def handleNewSignals(self):

        all_peaks_url = self.get_task_input_value("intensity_fine_filtered_data_url")
        if isinstance(all_peaks_url, str):
            self._allAlignedPeaksDataUrl.setText(all_peaks_url)

        strong_filtered_peaks_url = self.get_task_input_value(
            "intensity_filtered_data_url"
        )
        if isinstance(strong_filtered_peaks_url, str):
            self._inputStrongFilteredPeaksDataUrl.setText(strong_filtered_peaks_url)

        grains_data_url = self.get_task_input_value("indexed_grain_data_url")
        if isinstance(grains_data_url, str):
            self._inputGrainDataUrl.setText(grains_data_url)

        lattice_file = self.get_task_input_value("lattice_file")
        if isinstance(lattice_file, str):
            self._latticeParFile.setText(lattice_file)

        folder_file_config = self.get_task_input_value("folder_file_config")
        if isinstance(lattice_file, str):
            self.set_default_input("folder_file_config", folder_file_config)

    def handleSuccessfulExecution(self):
        if self.get_task_output_value("make_map_data_url", None) is None:
            return
        self._plot3DGrains(self.get_task_output_value("make_map_data_url"))

    def _plot3DGrains(self, url: str):
        self._refineGrainsPlot.getSceneWidget().clearItems()
        self._grainSphere = build_grain_spheres(url, self._grainSlider.value())
        self._refineGrainsPlot.getSceneWidget().addItem(self._grainSphere)

    def _updateGrainSize(self, size: float):
        if self._grainSphere:
            self._grainSphere.setRadiiNorm(radiiNorm=size)



from __future__ import annotations

from pathlib import Path

from ImageD11.unitcell import unitcell as UnitCell
from silx.gui import qt
from silx.gui.colors import Colormap
from silx.io.utils import DataUrl

from ewoks3dxrd.io import save_geometry_and_lattice_file
from ewoks3dxrd.nexus.peaks import read_peaks_attributes
from ewoks3dxrd.nexus.utils import group_exists
from ewoks3dxrd.tasks.filter_by_lattice import FilterByLattice, Inputs

from .common.ewoks3dxrd_widget import Ewoks3DXRDWidget
from .common.peak_filter_plot2d import PeakFilterPlot2D
from .common.process_name_line_edit import ProcessNameLineEdit
from .lattice.lattice_filtering_settings import LatticeFilteringSettings


class OWLatticeFilter(Ewoks3DXRDWidget, ewokstaskclass=FilterByLattice):
    name = "Lattice Filter"
    description = "Filter geometry transformed, segmented 3D Peaks."
    icon = "icons/filters-1.svg"
    _ewoks_inputs_to_hide_from_orange = (
        "reciprocal_dist_max",
        "reciprocal_dist_tol",
        "overwrite",
        "lattice_file",
        "process_group_name",
    )

    def __init__(self):
        super().__init__()
        self._settingsPanelWidget = qt.QWidget(self)
        settingLayout = qt.QVBoxLayout(self._settingsPanelWidget)
        self._settingsPanel = LatticeFilteringSettings(self)
        self._settingsPanel.sigPlotLatticeRings.connect(self._plotRings)
        self._ds_ring_limits = 1.5

        scrollArea = qt.QScrollArea(self)
        scrollArea.setWidgetResizable(True)
        scrollArea.setWidget(self._settingsPanel)
        settingLayout.addWidget(scrollArea)

        executeOverWriteLayout = qt.QFormLayout()
        self._geometryCorrectedUrl = qt.QLineEdit()
        self._processNameLineEdit = ProcessNameLineEdit(
            default=Inputs.model_fields["process_group_name"].default
        )
        self._overwrite = qt.QCheckBox()
        self._filterPeaksBtn = qt.QPushButton("Filter Peaks")
        executeOverWriteLayout.addRow(
            "Input: Geometry Transformed URL", self._geometryCorrectedUrl
        )
        executeOverWriteLayout.addRow(
            self._processNameLineEdit.label, self._processNameLineEdit
        )
        executeOverWriteLayout.addRow("Overwrite", self._overwrite)
        executeOverWriteLayout.addRow(self._filterPeaksBtn)
        settingLayout.addLayout(executeOverWriteLayout)
        self._filterPeaksBtn.clicked.connect(self.execute_ewoks_task)

        self.addControlWidget(self._settingsPanelWidget)

        self._plot = PeakFilterPlot2D(self)
        self._plot.setSizePolicy(qt.QSizePolicy.Expanding, qt.QSizePolicy.Expanding)
        self.addMainWidget(self._plot)
        self.registerInput(
            "geometry_updated_data_url",
            self._validateInputUrl,
            self._geometryCorrectedUrl.setText,
        )
        self.registerInput(
            "lattice_file", self._prepareLatticeFile, self._settingsPanel.setLatticeFile
        )
        self.registerInput(
            "reciprocal_dist_tol",
            self._settingsPanel.getReciprocalDistTol,
            self._settingsPanel.setReciprocalDistTol,
        )
        self.registerInput(
            "reciprocal_dist_max",
            self._settingsPanel.getReciprocalDistMax,
            self._settingsPanel.setReciprocalDistMax,
        )
        self.registerInput(
            "process_group_name",
            self._processNameLineEdit.getText,
            self._processNameLineEdit.setText,
        )
        self.registerInput(
            "overwrite", self._overwrite.isChecked, self._overwrite.setChecked
        )
        self._restoreDefaultInputs()
        self._plotIncomingPeaks()

    def _plotRings(self, limit: float | None = None):
        if limit is None:
            limit = self._ds_ring_limits
        else:
            self._ds_ring_limits = limit

        parms = self._settingsPanel.getLatticeParameters()
        try:
            unit_cell = UnitCell(
                lattice_parameters=parms["lattice_parameters"],
                symmetry=int(parms["lattice_space_group"]),
            )
            unit_cell.makerings(limit=limit)

            for i, ring_ds in enumerate(unit_cell.ringds):
                self._plot.addCurve(
                    [ring_ds, ring_ds], [-190, 190], legend=f"Data_{i}", linewidth=2
                )
            self._plot.setGraphXLabel("Reciprocal Distance (ds)")
            self._plot.setGraphYLabel("Azimuthal Angle (eta)")
        except Exception as e:
            self.showError(e, title="Lattice Ring Plot")

    def handleNewSignals(self):
        input_peaks_url = self.get_task_input_value("geometry_updated_data_url", None)
        if input_peaks_url is not None:
            self._geometryCorrectedUrl.setText(input_peaks_url)
            self._plotIncomingPeaks()

    def _plotIncomingPeaks(self):
        input_url = self.get_task_input_value("geometry_updated_data_url", None)
        if input_url is None:
            return
        data_url = DataUrl(input_url)
        nexus_file_path, data_group_path = (
            data_url.file_path(),
            data_url.data_path(),
        )
        incoming_3d_peaks = read_peaks_attributes(
            filename=nexus_file_path,
            process_group=data_group_path,
        )
        self._plot.addUnfilteredScatter(
            x=incoming_3d_peaks["ds"],
            y=incoming_3d_peaks["eta"],
            value=len(incoming_3d_peaks["eta"]) * [1],
            colormap=Colormap(name="viridis", autoscaleMode="percentile_1_99"),
        )
        self._plot.setGraphXLabel("Reciprocal distance (ds)")
        self._plot.setGraphYLabel("Sum Intensity")
        self._plot.resetZoom()

    def handleSuccessfulExecution(self):
        if self.get_task_output_value("lattice_filtered_data_url", None) is None:
            return

        data_url = DataUrl(self.get_task_output_value("lattice_filtered_data_url"))
        nexus_file_path, lattice_data_group_path = (
            data_url.file_path(),
            data_url.data_path(),
        )
        lattice_3d_peaks = read_peaks_attributes(
            filename=nexus_file_path,
            process_group=lattice_data_group_path,
        )
        self._plotRings(limit=max(lattice_3d_peaks["ds"]))
        self._plot.addFilteredScatter(
            x=lattice_3d_peaks["ds"],
            y=lattice_3d_peaks["eta"],
            value=lattice_3d_peaks["Number_of_pixels"],
            colormap=Colormap(
                name="viridis", normalization="log", autoscaleMode="percentile_1_99"
            ),
        )
        self._plot.setGraphXLabel("Reciprocal distance (ds)")
        self._plot.setGraphYLabel("Azimuthal angle (eta)")
        self._plot.resetZoom()

    def _prepareLatticeFile(self) -> str:
        file_path = Path(DataUrl(self._geometryCorrectedUrl.text()).file_path())
        params = self._settingsPanel.getLatticeParameters()
        assert params["lattice_name"] is not None
        assert len(params["lattice_parameters"]) == 6
        assert params["lattice_space_group"] is not None
        lattice_name: str = params["lattice_name"]
        lattice_file = file_path.parent / f"{lattice_name}.par"
        save_geometry_and_lattice_file(
            file_path=lattice_file,
            geom_dict={},
            lattice_dict={
                "lattice_parameters": params["lattice_parameters"],
                "lattice_space_group": params["lattice_space_group"],
            },
        )
        return str(lattice_file)

    def _validateInputUrl(self) -> str:
        input_peaks_url = DataUrl(self._geometryCorrectedUrl.text().strip())
        group_exists(
            filename=input_peaks_url.file_path(),
            data_group_path=input_peaks_url.data_path(),
        )
        return input_peaks_url.path()



from __future__ import annotations

import math
from pathlib import Path
from typing import Any

import numpy as np
import qtawesome
from silx.gui import qt
from silx.gui.colors import Colormap
from silx.gui.qt import QSizePolicy
from silx.io.utils import DataUrl

from ewoks3dxrd.io import get_frame_image, get_monitor_scale_factor_for_frame_index
from ewoks3dxrd.nexus.peaks import read_peaks_attributes
from ewoks3dxrd.segment import segment_frame
from ewoks3dxrd.tasks.segment_scan import SegmentScan

from .common.ewoks3dxrd_plot2d import Ewoks3DXRDPlot2D
from .common.ewoks3dxrd_widget import Ewoks3DXRDWidget
from .segment.dual_plot_silx import SilxSyncDualPlot
from .segment.segmenter_settings import SegmenterSettings
from .segment.utils import ask_confirmation_to_repeat_segmentation


class OWFrameSegmenter(Ewoks3DXRDWidget, ewokstaskclass=SegmentScan):
    name = "Peaks Segmentation"
    description = "Runs segmentation on a scan, with preview on a frame."
    icon = "icons/filter_frames.svg"
    _ewoks_inputs_to_hide_from_orange = (
        "overwrite",
        "folder_config",
        "segmenter_algo_params",
        "correction_files",
        "monitor_name",
    )

    def __init__(self):
        super().__init__()

        _settingsPanelWidget = qt.QWidget(self)
        settingLayout = qt.QVBoxLayout(_settingsPanelWidget)

        self._settingsPanel = SegmenterSettings(self)
        self._settingsPanel.sigParametersChanged.connect(self._displayFrame)
        scrollArea = qt.QScrollArea(self)
        scrollArea.setWidgetResizable(True)
        scrollArea.setWidget(self._settingsPanel)
        settingLayout.addWidget(scrollArea)
        self._settingsPanel.sigParametersChanged.connect(self._displayFrame)

        segmentProgressOverWriteLayout = qt.QFormLayout()
        self._overwriteCheckbox = qt.QCheckBox()
        self._progressBar = qt.QProgressBar()
        self._progressBar.setRange(0, 100)
        self._progressBar.setValue(0)
        segmentProgressOverWriteLayout.addRow(
            "Overwrite Segmentation Result", self._overwriteCheckbox
        )
        segmentProgressOverWriteLayout.addRow(self._progressBar)
        settingLayout.addLayout(segmentProgressOverWriteLayout)

        btnLayout = qt.QHBoxLayout()
        self._executeScanSegmentationBtn = qt.QPushButton("Execute Segment Scan")
        self._executeScanSegmentationBtn.clicked.connect(self._executeEwoksTask)
        self._executeScanSegmentationBtn.setSizePolicy(
            qt.QSizePolicy.Preferred, qt.QSizePolicy.Fixed
        )
        btnLayout.addWidget(self._executeScanSegmentationBtn)
        self._showSegmentationPlotBtn = qt.QPushButton(qtawesome.icon("fa6.eye"), None)
        self._showSegmentationPlotBtn.setFlat(True)
        self._showSegmentationPlotBtn.setToolTip("Show 3D segmented result")
        self._showSegmentationPlotBtn.clicked.connect(self._plotSegmentationOutput)
        self._showSegmentationPlotBtn.setDisabled(True)
        self._showSegmentationPlotBtn.setSizePolicy(
            qt.QSizePolicy.Fixed, qt.QSizePolicy.Preferred
        )
        btnLayout.addWidget(self._showSegmentationPlotBtn)
        settingLayout.addLayout(btnLayout)

        self._lastParams = None

        self._plotWidget = SilxSyncDualPlot(self)
        self._plotWidget.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)

        self.addControlWidget(_settingsPanelWidget)
        self.addMainWidget(self._plotWidget)
        self.registerInput(
            "folder_config",
            self._settingsPanel.getFolderConfig,
            self._settingsPanel.setFolderConfig,
        )
        self.registerInput(
            "segmenter_algo_params",
            self._settingsPanel.getSegmenterConfig,
            self._settingsPanel.setSegmenterConfig,
        )
        self.registerInput(
            "correction_files",
            self._settingsPanel.getCorrectionFiles,
            self._settingsPanel.setCorrectionFiles,
        )
        self.registerInput(
            "monitor_name",
            self._settingsPanel.getMonitorName,
            self._settingsPanel.setMonitorName,
        )
        self.registerInput(
            "overwrite",
            self._overwriteCheckbox.isChecked,
            self._overwriteCheckbox.setChecked,
        )
        self._restoreDefaultInputs()

    def _displayFrame(self, params: dict | None):
        if not params:
            return

        raw_image, scale_factor = self._getRawImage(params)
        try:
            smoothed_img, peak_pos = segment_frame(raw_image, params, scale_factor)
        except Exception as e:
            self.showError(e)
            return

        self._plotWidget.setLeftImage(
            array2D=raw_image,
            title=f"Raw Image. {len(peak_pos[0])} peaks found",
            xLabel="f_raw",
            yLabel="s_raw",
        )
        self._plotWidget.setRightImage(
            array2D=smoothed_img,
            title="Background Corrected Image",
            xLabel="f_raw",
            yLabel="s_raw",
        )
        self._plotWidget.setRightScatter(x=peak_pos[0], y=peak_pos[1])

    def _getRawImage(self, params: dict) -> tuple[np.ndarray, float | None]:
        masterfile_path = Path(self._settingsPanel.getMasterFilePath())
        frame_idx = self._settingsPanel.getFrameIdx()
        raw_image = get_frame_image(
            file_path=masterfile_path,
            detector=params["file_folders"]["detector"],
            scan_id=str(self._settingsPanel.getScanNumber()) + ".1",
            frame_idx=frame_idx,
        ).astype("uint16")

        monitor_name = params["monitor_name"]
        if monitor_name is None:
            return raw_image, None

        scale_factor = get_monitor_scale_factor_for_frame_index(
            masterfile_path=masterfile_path,
            scan_number=str(self._settingsPanel.getScanNumber()),
            detector=params["file_folders"]["detector"],
            monitor_name=monitor_name,
            frame_idx=frame_idx,
        )
        return raw_image, scale_factor

    def _getParameters(self) -> dict[str, Any]:
        params = self._settingsPanel.getParameters()
        params = {
            **params,
            "overwrite": self._overwriteCheckbox.isChecked(),
        }
        return params

    def _executeEwoksTask(self):
        try:
            params = self._getParameters()
            if params == self._lastParams:
                decision = ask_confirmation_to_repeat_segmentation()
                if decision == "cancel":
                    return
                elif decision == "show":
                    self._plotSegmentationOutput()
                    return
                elif decision == "continue":
                    pass
                else:
                    raise ValueError(f"Unknown decision {decision}")
        except Exception as e:
            self.showError(e)
            return

        self.execute_ewoks_task()

    def handleSuccessfulExecution(self):
        self._lastParams = self._getParameters()
        self._plotSegmentationOutput()
        self._showSegmentationPlotBtn.setEnabled(True)
        self.progressBarSet(100)

    def _plotSegmentationOutput(self):
        outputs = self.get_task_output_values()
        data_url = DataUrl(outputs["segmented_peaks_url"])
        nexus_file_path = data_url.file_path()
        segmented_data_group_path = data_url.data_path()

        segmented_3d_peaks = read_peaks_attributes(
            filename=nexus_file_path,
            process_group=segmented_data_group_path,
        )
        plot = Ewoks3DXRDPlot2D()
        plot.addScatter(
            x=segmented_3d_peaks["f_raw"],
            y=segmented_3d_peaks["s_raw"],
            value=segmented_3d_peaks["omega"],
            legend="3D Segmented Peaks",
            symbol="x",
            colormap=Colormap(
                name="viridis",
            ),
        )
        plot.setGraphXLabel("f_raw")
        plot.setGraphYLabel("s_raw")
        plot.setGraphTitle("3D Segmented Peaks")
        plot.resetZoom()
        plot.show()

    def progressBarSet(self, value: int):
        self._progressBar.setValue(int(math.ceil(value)))

    def progressBarInit(self):
        self._progressBar.setValue(0)
